#========================================================================================
# package      DE_EPAGES::Core::API::CSV
# state        public
# base
#----------------------------------------------------------------------------------------
# description  Provides methods for creating and parsing CSV (comma separated values)
#               files.
#               This module wraps Text::CSV_XS and support for unicode by default.
#               Therefore the Text::CSV_XS::new option "binary" is always on unless
#               explicitely disabled. All function arguments and return values are
#               assumed to be native UTF-8 perl strings.
#----------------------------------------------------------------------------------------
# example      $aFields = ParseCSV( '/Shops/DemoShop/Products/"s k u"', { 'sep_char' => '/' } );
# example      my $aFields = ['Shops', 'DemoShop', Products', 's k u'];
#               print CombineCSV( $aFields, { 'sep_char' => '/' } );
# example      my $CSV = DE_EPAGES::Core::API::CSV->new;
#               my $fh = FileHandle->new;
#               $fh->open( $FileName, '<:encoding(utf-8)' ) or die "Can't read $FileName: $!";
#               while( $aFields = $CSV->getline($fh) ) {
#                   # process record
#               }
#========================================================================================
package DE_EPAGES::Core::API::CSV;
use base qw( Exporter );

use strict;
use Encode qw( encode_utf8 );
use IO::Scalar;
use Text::CSV_XS;

use DE_EPAGES::Core::API::String qw( CheckForNonPrintableChars );
use DE_EPAGES::Core::API::TestParameter qw( TestArray TestHash );
use DE_EPAGES::Core::API::Error qw ( Error );

our @EXPORT_OK = qw(
    ParseCSV
    CombineCSV
    ParseCSVArray
    CombineCSVArray
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'GET_LINE_FAILED' => '#Message',
    }
}
#ERRORDEFINITION

#========================================================================================
# function     ParseCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFields = ParseCSV( $Line, $hOptions );
# syntax       $aFields = ParseCSV( $Line );
#----------------------------------------------------------------------------------------
# description  Parses one CSV formatted line and returns the parsed fields.
#----------------------------------------------------------------------------------------
# input        $Line | CSV formatted string | string
# input        $hOptions | see <function new> | ref.hash.string
# return       $aFields | parsed fields | ref.array.string
#========================================================================================
sub ParseCSV($;$) {
    my( $Line, $hOptions ) = @_;
    $hOptions = {} unless defined $hOptions;
    $Line = '' unless defined $Line;
    TestHash( 'hOptions', $hOptions );

    return __PACKAGE__->new( $hOptions )->parse( $Line );
}

#========================================================================================
# function     CombineCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Line = CombineCSV( $aFields, $hOptions );
# syntax       $Line = CombineCSV( $aFields );
#----------------------------------------------------------------------------------------
# description  Combines a list of fields to one CSV formatted line.
#----------------------------------------------------------------------------------------
# input        $aFields | parsed fields | ref.array.string
# input        $hOptions | (optional) see <function new> | ref.hash.string
# return       $Line | CSV formatted string | string
#========================================================================================
sub CombineCSV($;$) {
    my( $aFields, $hOptions ) = @_;
    $hOptions = {} unless defined $hOptions;
    TestArray( 'aFields', $aFields );
    TestHash( 'hOptions', $hOptions );

    return __PACKAGE__->new( $hOptions )->combine( $aFields );
}

#========================================================================================
# function     ParseCSVArray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aaFields = ParseCSVArray( $Content, $hOptions );
# syntax       $aaFields = ParseCSVArray( $Content );
# example      $aaFields = ParseCSVArray( "Name,Value\nSmith,45\nSummers,21\n" );
#----------------------------------------------------------------------------------------
# description  Combines a list of fields to a CSV formatted string.
#----------------------------------------------------------------------------------------
# input        $Content | CSV formatted string | string
# input        $hOptions | (optional) see <function new>.
#                           The default end-of-line string (eol) is "\n".
#                           Other eol strings don't work (see perldoc Text::CSV_XS).
#                         | ref.hash.string
# return       $aaFields | parsed fields | ref.array.array.string
#========================================================================================
sub ParseCSVArray($;$) {
    my ($Content, $hOptions) = @_;
    $hOptions = {} unless defined $hOptions;
    $hOptions->{'eol'} = "\n" unless exists $hOptions->{'eol'};
    TestHash( 'hOptions', $hOptions );

    my $CSV = DE_EPAGES::Core::API::CSV->new( $hOptions );
    my $fh = IO::Scalar->new( \$Content );
    my @Lines;
    while( my $aFields = $CSV->getline($fh) ) {
        push @Lines, [ map { encode_utf8($_) } @$aFields ];
    }

    return \@Lines;
}

#========================================================================================
# function     CombineCSVArray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Line = CombineCSVArray( $aaFields, $hOptions );
# syntax       $Line = CombineCSVArray( $aaFields );
#----------------------------------------------------------------------------------------
# description  Combines a list of lines to one CSV formatted string.
#----------------------------------------------------------------------------------------
# input        $aaFields | array of lines, each line is an array of strings
#                         | ref.array.array.string
# input        $hOptions | see <function new>
#                           The default end-of-line string (eol) is "\n".
#                         | ref.hash.string
# return       $Line | CSV formatted string | string
#========================================================================================
sub CombineCSVArray($;$) {
    my( $aLines, $hOptions ) = @_;
    $hOptions = {} unless defined $hOptions;
    $hOptions->{'eol'} = "\n" unless exists $hOptions->{'eol'};
    TestArray( 'aLines', $aLines );
    TestHash( 'hOptions', $hOptions );

    my $CSV = DE_EPAGES::Core::API::CSV->new( $hOptions );
    return join( '', map { $CSV->combine( $_ ) } @$aLines );
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CSV = DE_EPAGES::Core::API::CSV->new( $hOptions );
# syntax       $CSV = DE_EPAGES::Core::API::CSV->new;
# example      $CSV = DE_EPAGES::Core::API::CSV->new( { 'sep_char' => '/' } );
#----------------------------------------------------------------------------------------
# description  Create a CSV parser and formatter object.
#----------------------------------------------------------------------------------------
# input        $hOptions | (optional) format options, see also Text::CSV_XS::new.
#                       <ul>
#                           <li>sep_char - seperator character (default: ',')</li>
#                           <li>binary - allow embedded newlines and unicode (default: 1)</li>
#                           <li>quote_char - quote character (default: '"')</li>
#                           <li>escape_char - escape character (default: '"')</li>
#                           <li>eol - line break character(s) (default: '')</li>
#                           <li>always_quote - always quote fields (default: 0)<li>
#                       </ul>
# return       $CSV | CSV parser and formatter | object
#========================================================================================
sub new {
    my ($class, $hOptions) = @_;

    # allow only documented options
    my %options = ( 'binary' => 1 );
    foreach my $key (qw( sep_char binary quote_char escape_char eol always_quote )) {
        next unless exists $hOptions->{$key};
        $options{$key} = $hOptions->{$key};
    }

    return bless {
        Converter => Text::CSV_XS->new(\%options)
    }, $class;
}

#========================================================================================
# function     converter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Converter = $CSV->converter;
#----------------------------------------------------------------------------------------
# description  Returns the underlying Text::CSV_XS object.
#----------------------------------------------------------------------------------------
# return       $Converter | instance of Text::CSV_XS | object
#========================================================================================
sub converter {
    return shift->{Converter};
}

#========================================================================================
# function     parse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFields = $CSV->parse( $Line );
#----------------------------------------------------------------------------------------
# description  Parses one CSV formatted line and returns the parsed fields.
#----------------------------------------------------------------------------------------
# input        $Line | CSV formatted string | string
# return       $aFields | parsed fields | ref.array.string
#========================================================================================
sub parse {
    my $self = shift;
    my ($Line) = @_;

    return [ '' ] unless defined $Line;
    return [ '' ] if $Line eq '';

    utf8::upgrade($Line) unless utf8::is_utf8($Line);

    my $Converter = $self->converter;
    $Converter->parse( $Line );

    return [ $Converter->fields ];
}

#========================================================================================
# function     combine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Line = $CSV->combine( $aFields );
#----------------------------------------------------------------------------------------
# description  Combines a list of fields to one CSV formatted line.
#----------------------------------------------------------------------------------------
# input        $aFields | parsed fields | ref.array.string
# return       $Line | CSV formatted string | string
#========================================================================================
sub combine {
    my $self = shift;
    my ($aFields) = @_;
    TestArray( 'aFields', $aFields );

    my $Converter = $self->converter;
    $Converter->combine( @$aFields );

    return $Converter->string;
}

#========================================================================================
# function     getline
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFields = $CSV->getline( $FileHandle );
#----------------------------------------------------------------------------------------
# description  Reads one record from a CSV file.
#               Returns the parsed fields or undef on end of file.
#----------------------------------------------------------------------------------------
# input        $FileHandle | file handle | IO::Handle object
# return       $aFields | parsed fields, undef on EOF | ref.array.string
#========================================================================================
sub getline {
    my $self = shift;
    my ($FileHandle) = @_;

    my $aFields = $self->converter->getline( $FileHandle );

    # check for non printable characters, throw a NON_PRINTABLE_CHARS error if check is positive
    if (defined $aFields) {
        CheckForNonPrintableChars(join $self->converter->sep_char, @$aFields);
    }

    ##no critic (ProhibitMagicNumbers)
    if( !defined $aFields ) {
        if( $Text::CSV_XS::VERSION >= 0.33 ) {
            return undef if $self->converter->eof;
        }

        my $Message = $Text::CSV_XS::VERSION < 0.33 ? $! : $self->converter->error_diag();
        Error('GET_LINE_FAILED', {
            'Message' => $Message,
            $self->formatSpec,
        });
    }

    # old version of eof
    if( $Text::CSV_XS::VERSION < 0.33 ) {
        return undef if 0 == scalar @$aFields;
    }
    ##use critic (ProhibitMagicNumbers)

    return $aFields;
}

#========================================================================================
# function     formatSpec
# state        public
#----------------------------------------------------------------------------------------
# syntax       %Format = $CSV->formatSpec;
#----------------------------------------------------------------------------------------
# description  Returns the current format specification.
#----------------------------------------------------------------------------------------
# return       %Format | current format specificationm, see <function new> | hash.string
#========================================================================================
sub formatSpec {
    my $self = shift;

    my $Converter = $self->converter;
    return (
        'sep_char' => $Converter->sep_char,
        'quote_char' => $Converter->quote_char,
        'escape_char' => $Converter->escape_char,
        'eol' => $Converter->eol,
        'binary' => $Converter->binary,
    );
}

1;
