#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentDetails
# state        public
# description  ShipmentDetails element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentDetails;

use Date::Format qw ( time2str );

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentItem;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Service;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Notification;

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper;
use DE_EPAGES::DHLIntraship::API::Constants qw (
    $EKP_LENGHT
    $PRODUCTCODE_LENGHT
    %MAPPING_PRODUCTCODE_PRODUCTNAME
);

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShipmentDetails = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentDetails->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $ShipmentDetails | DHL Versenden shipment details element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShipmentDetails = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with shipment details data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hShipmentDetails | DHL Versenden shipment details element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $ShippingFulfillmentMethod = $ShippingLabel->get('ShippingFulfillmentMethod');
    my $CustomerAddress = $ShippingLabel->get('Address');

    my $LeafNodeHelper = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper->new();
    $LeafNodeHelper->addLeaf('accountNumber', $self->_getAccountNumber($ShippingFulfillmentMethod, $CustomerAddress));

    my @Warnings = @{$LeafNodeHelper->getWarnings()};

    my $hShipmentItem = $self->_assembleShipmentItem($ShippingLabel);
    my $hService = $self->_assembleService($ShippingLabel);
    my $hNotification = $self->_assembleNotification($CustomerAddress);

    push(@Warnings, @{$hShipmentItem->{'nodeWarnings'}});
    push(@Warnings, @{$hService->{'nodeWarnings'}});
    push(@Warnings, @{$hNotification->{'nodeWarnings'}});

    return {
        'nodeData' => {
            'ShipmentDetails' => {
                'product' => $self->_getProductCode($ShippingFulfillmentMethod, $CustomerAddress),
                'accountNumber' => $LeafNodeHelper->getLeaf('accountNumber'),
                'customerReference' => $ShippingLabel->alias(),
                'shipmentDate' => $self->_getShipmentDate($ShippingLabel),
                # returnShipmentAccountNumber (optional. field to include again the DHL account number)
                # returnShipmentReference (optional.A reference number for better association purposes. It appears on return)
                %{$hShipmentItem->{'nodeData'}},
                %{$hService->{'nodeData'}},
                # no optional BankData (we don't use it)
            }
        },
        'nodeWarnings' => \@Warnings
    };
}

#========================================================================================
# Local methods
#========================================================================================
sub _getAccountNumber {
    my $self = shift;
    my ($ShippingFulfillmentMethod, $CustomerAddress) = @_;

    return DE_EPAGES::DHLIntraship::API::Tools->GetCustomerNumber($ShippingFulfillmentMethod, $CustomerAddress);
}

#========================================================================================
sub _getProductCode {
    my $self = shift;
    my ($ShippingFulfillmentMethod, $CustomerAddress) = @_;

    my $AccountNumber = $self->_getAccountNumber($ShippingFulfillmentMethod, $CustomerAddress);
    my $ProductCodeID = substr($AccountNumber, $EKP_LENGHT, $PRODUCTCODE_LENGHT);

    return $MAPPING_PRODUCTCODE_PRODUCTNAME{$ProductCodeID};
}

#========================================================================================
sub _getShipmentDate {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $EstimatedDeliveryDate = $ShippingLabel->get('EstimatedDeliveryDate');

    if ( defined $EstimatedDeliveryDate ) {
        return time2str('%Y-%m-%d', $EstimatedDeliveryDate->epoch);
    }
    return '';
}

#========================================================================================
sub _assembleShipmentItem {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentItem->assemble($ShippingLabel);
}

#========================================================================================
sub _assembleService {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Service->assemble($ShippingLabel);
}

#========================================================================================
sub _assembleNotification {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Notification->assemble($Address);
}

1;