use strict;
use warnings;
use Getopt::Long;
use Time::HiRes qw( time );

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use DE_EPAGES::GoogleBase::API::Cartridge;
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::ELogistics::API::Tools qw (
    IsValidAddressCountry
);
use Data::Dumper;

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }

    unless ( defined $ShopAlias ) {
        print STDERR "Missing parameter -shop.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            if ( defined $ShopAlias and $ShopAlias ne 'all' ) {
                my $Shop;
                eval { $Shop = LoadObjectByPath('/Shops')->child($ShopAlias); };
                if ( ExistsError() ) {
                    printf STDERR ( "[%s] WARN: Shop '%s' does not exist.\n", GetError() );
                    return;
                }

                my $ShopAddr = $Shop->get('Address');

                my $ShippingFullfilmentMethods = $Shop->get('ShippingFulfillmentMethods');
                my $ELogisticsFound = 0;
                foreach my $Method (@{$ShippingFullfilmentMethods}) {
                    if ($Method->class()->isSubClassOf('ShippingFulfillmentMethodELogistics'))
                    {
                        $ELogisticsFound = 1;
                        my $WebserviceLicenceNumber = $Method->get('WebserviceLicenceNumber');
                        defined $WebserviceLicenceNumber ? print "WebserviceLicenceNumber: ".$Method->get('WebserviceLicenceNumber')."\n" : print "WebserviceLicenceNumber empty !!!\n";
                        my $WebserviceUsername = $Method->get('WebserviceUsername');
                        defined $WebserviceUsername ? print "WebserviceUsername: ".$Method->get('WebserviceUsername')."\n" : print "WebserviceUsername empty !!!\n";
                        print "IsValidAddressCountry: ".IsValidAddressCountry($ShopAddr)."\n";
                        my $WebservicePassword = $Method->get('WebservicePassword');
                        defined $WebservicePassword ? print "WebservicePassword exists" : print "WebserviceUsername empty !!!\n";
                        _mandatoryPickPostFieldsVisible( $Shop );

                    }
                }
                if (! $ELogisticsFound) {
                    print "No ShippingFulfillmentmethod E-Logistics found in shop $ShopAlias !!!\n" if ! $ELogisticsFound;
                    return;
                }
            }

            eval { SynchronizeCache(); };
            LogError(GetError()) if ExistsError();

            return;
        },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub _mandatoryPickPostFieldsVisible {
    my ( $Shop ) = @_;

    my $RegistrationFields = $Shop->get('RegistrationField');
    foreach my $Alias ( qw( FirstName LastName AddressExtension Zipcode City CountryID )) {
        my $RegistrationField = $RegistrationFields->{$Alias};
        print "$RegistrationFields->{$Alias}" if ! $RegistrationField;
        return 0 if ! $RegistrationField;
        print  "Address field $Alias not set to visible!!!\nMandatoryPickPostFieldsVisible: 0\n" if ! $RegistrationField->{'IsShippingAddressVisible'};
        return 0 if ! $RegistrationField->{'IsShippingAddressVisible'};
    }
    print "MandatoryPickPostFieldsVisible: 1 \n";
    return 1;
}

sub usage {
    print <<END_USAGE;
Description:
    Prints all needed information for Elogistics to be usable in Shop.
Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shop  name of shop                                   ()
flags:
    -help       show the command line options
Example:
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);
