#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts
# state        public
# base         DE_EPAGES::Presentation::API::Object::Pager
# description  Pager for all products from Google Merchant Center.
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts;
use base DE_EPAGES::Presentation::API::Object::Pager;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use JSON::MaybeXS qw ( encode_json );
use POSIX qw ( ceil );

use constant REGEX_EXTRACT_ALIAS_FROM_LANG_EPAGES_FORMAT => qr/(.+)_[a-z]{2}_[A-Z]{2}$/mp;

sub new {
    my ( $class, $hInfo ) = @_;

    $hInfo->{'MCItems'} = [];
    $hInfo->{'MCItemCount'} = 0;
    my $self = $class->SUPER::new($hInfo);

    return $self;
}


#========================================================================================
# function     execute
# state        private
#----------------------------------------------------------------------------------------
# syntax      $Pager->execute;
#----------------------------------------------------------------------------------------
# description  Executes the request on Google MerchantCenter - Content API
#----------------------------------------------------------------------------------------
# return       none.
#========================================================================================
sub execute {
    my $self = shift;
    my $GoogleApiService = $self->get('GoogleApiService');

    my $Page = $self->get('Page');
    my $Object = $self->get('Object');
    my $Shop = $Object->getSite();

    if ($Shop->isUnity()) {
        $self->{'Attributes'}->{'ViewAction'} = 'UnityMBO-ViewGoogleSmartShoppingProductFeed';
    }
    my $ResultsPerPage = $self->get('PageSize');
    my $OrderBy;
    my $OrderDesc;
    my $Result;
    eval {
        $Result = $GoogleApiService->productsStatuses($Page, $ResultsPerPage, $OrderBy, $OrderDesc);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Request products statuses failed', { 'Error' => $Error->message() });
    }

    my $CountResults = defined($Result->{'results'}) ? $Result->{'results'} : 0;
    $self->set({
         'PagerSize' => ceil ($CountResults / $ResultsPerPage),
         'MCItems' => $Result->{'items'},
         'MCItemCount' => $CountResults,
    });
}

#========================================================================================
# function     countItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Count = $Pager->countItems;
#----------------------------------------------------------------------------------------
# description  Returns the total number of products.
#----------------------------------------------------------------------------------------
# return       $Count | number of items | integer
#========================================================================================
sub countItems {
    my $self = shift;

    return $self->get('MCItemCount');
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aItems = $Pager->items($PageNumber);
#----------------------------------------------------------------------------------------
# description  Returns all objects of one page.
# input        $PageNumber | defines the actual page of the pager
#----------------------------------------------------------------------------------------
# return       $aItems | list of objects | ref.array.object
#========================================================================================
sub items {
    my $self = shift;

    my $Object = $self->get('Object');
    my $Shop = $Object->getSite();
    my $MCItems = $self->get('MCItems');
    if($self->countItems() > 0) {
        foreach my $MCItem (@{$MCItems}) {
            my $ItemAliasWithLang = $self->_extractAliasWithLangFromExternalProductId($MCItem->{'productId'});
            $MCItem->{'ShopProduct'} = undef;
            if ($ItemAliasWithLang =~ REGEX_EXTRACT_ALIAS_FROM_LANG_EPAGES_FORMAT) {
                # Example offerId:  "12345_de_DE"
                # becomes: "12345"
                $MCItem->{'ShopProduct'} = $self->_getProductByAlias($Shop, $1);
            }
            $MCItem->{'issues_json'} = (scalar $MCItem->{'issues'}) ? encode_json($MCItem->{'issues'}) : {};
            $MCItem->{'ProductName'} = ($MCItem->{'ShopProduct'}) ? $MCItem->{'ShopProduct'}->{'NameOrAlias'}: $MCItem->{'productId'};
        }
    }

    return $MCItems;
}

sub _getProductByAlias {
    my $self = shift;
    my ($Shop, $ProductAlias) = @_;

    return unless isValid($ProductAlias);
    return unless $Shop->folder('Products')->existsChild($ProductAlias);

    my $Product = $Shop->folder('Products')->child($ProductAlias);
    my $hValues = $Product->get([ qw (ID Alias NameOrAlias ImageThumbnail)]);

    if (defined $hValues->{'ID'}) {
        return {
            "ID"             => $hValues->{'ID'},
            "Alias"    => $hValues->{'Alias'},
            "NameOrAlias"    => $hValues->{'NameOrAlias'},
            "Product"        => $Product
        }
    }

}


# Example offerId:  "online:de:12345_de_DE"
# Return: "12345_de_DE"
sub _extractAliasWithLangFromExternalProductId {
    my $self = shift;
    my ($ExternalProductId) = @_;

    my @OfferParts = split(/:/, $ExternalProductId);
    my $AliasWithLang = pop @OfferParts;

    return $AliasWithLang;
}

1;
