#========================================================================================
# package      DE_EPAGES::MetaCatalog::API::CSV::MetaCatalogProductsExportDriver
# base         DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver Exporter
# state        private
#----------------------------------------------------------------------------------------
# description  Export driver for the Meta Catalog CSV feed
#
#               https://www.facebook.com/business/help/120325381656392?id=725943027795860
#========================================================================================
package DE_EPAGES::MetaCatalog::API::CSV::MetaCatalogProductsExportDriver;
use base qw ( DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver Exporter);

use strict;
use DE_EPAGES::Product::API::Product qw ( GetUnityPrettyUrl );
use DE_EPAGES::Core::API::Error                     qw (ExistsError);
use DE_EPAGES::Object::API::Factory                 qw (LoadObject);
use DE_EPAGES::ProductPortal::API::Export qw ( IsProductValidPortalExt1 );

# Columns to export
#========================================================================================
my @MAP_META_ATTRIBUTES = qw (
    id
    title
    description
    availability
    condition
    price
    link
    image_link
    brand

    item_group_id
    shipping
    shipping_weight
);
#========================================================================================


#========================================================================================
# function     exportCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportCSV($FileName, $hFileFormat, $aColumnNames, $ViewObject, $aObjects);
#----------------------------------------------------------------------------------------
# description  starts the csv export
#----------------------------------------------------------------------------------------
# input        $FileName | export to file | string
# input        $hFileFormat | file format <function DE_EPAGES::Presentation::API::CSV::MonitoredExportDriver::exportCSV> | ref.hash
# input        $aColumnNames | name of column | ref.array.string
# input        $ViewObject | export based on this object | PortalConfig
# input        $aObjects | objects to export | ref.array.object
#========================================================================================
sub exportCSV {
    my $self = shift;
    my ($FileName, $hFileFormat, $aColumnNames, $ViewObject, $aObjects) = @_;
    $hFileFormat = {} unless defined $hFileFormat;
    $hFileFormat->{'sep_char'}     = ',' if not defined $hFileFormat->{'sep_char'};
    $hFileFormat->{'always_quote'} = '1' if not defined $hFileFormat->{'always_quote'};
    $hFileFormat->{'encoding'}     = 'utf8' unless defined $hFileFormat->{'encoding'};

    if ( $ViewObject->get('ExportViaPixel') ) {
        return;
    }

    #-- Add some custom variables and export multiple times, once per languageID
    my $Shop = $ViewObject->getSite();
    my $Portal = $ViewObject->get('Portal');
    my $ahLanguages = $Shop->get('Languages');
    foreach my $hLanguageInfo ( @$ahLanguages ) {
        my $Code2 = $hLanguageInfo->{'Code2'};
        my $LanguageID = $hLanguageInfo->{'LanguageID'};

        my $CurrencyID = $Shop->get('CurrencyID');

        $self->{'MetaLanguageID'} = $LanguageID;
        $self->{'MetaCurrencyID'} = $CurrencyID;

        #-- Normally, it would just be the GUID, but we add the language code onto the end
        $FileName = $Portal->get('ExportPublicPath') . '/' . $ViewObject->get('GUID') . '_' . $Code2 . '.csv';

        $self->SUPER::exportCSV($FileName, $hFileFormat, [@MAP_META_ATTRIBUTES], $ViewObject, $aObjects);

        #-- Reset the monitor count for the next iteration of the loop/export
        $self->{'Monitor'}->set({ 'Current' => 0 }) if $self->{'Monitor'};
    }

    return;
}

#-- Base package sets {'LanguageID'} and {'CurrencyID'} and we don't want to touch it because it is the base package
#-- So instead overload these subroutines to return our own language/currency id
sub languageID { return shift->{'MetaLanguageID'}; }
sub currencyID { return shift->{'MetaCurrencyID'}; }

sub id {
    my $self = shift;
    my ($Product) = @_;

    return substr($Product->alias, 0, 100);
}

sub title {
    my $self = shift;
    my ($Product) = @_;

    my $ProductName = $Product->getWithParent('Name', $self->languageID );
    return substr($ProductName, 0, 200);
}

#-- No HTML apparently
sub description {
    my $self = shift;
    my ($Product) = @_;

    my $Description = $self->removeHtmlTags($Product->getWithParent( $self->descriptionAttribute->alias, $self->languageID ));
    return substr($Description, 0, 9999);
}

sub availability {
    my $self = shift;
    my ($Product) = @_;

    my $StockLevel = $Product->get('StockLevel');
    if ( defined $StockLevel && $StockLevel <= 0 ) {
        return 'out of stock';
    }
    
    return 'in stock';
}

sub condition {
    my $self = shift;
    my ($Product) = @_;

    return 'new';
}

sub price {
    my $self = shift;
    my ($Product) = @_;

    return $self->productPrice($Product) . ' ' . $self->currencyID;
}

sub link {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getSite()->get('IsUnityRegistered') ? GetUnityPrettyUrl($Product, $self->languageID, $self->languageID, 1) : $self->productUrl($Product);
}

sub image_link {
    my $self = shift;
    my ($Product) = @_;

    return $self->imageUrl($Product, $self->imageAttribute);
}

sub brand {
    my $self = shift;
    my ($Product) = @_;

    my $Brand = $Product->getWithParent('Manufacturer', $self->languageID);
    return substr($Brand, 0, 100);
}
#-- Everything above here is mandatory

#-- Everything below here is optional
sub item_group_id {
    my $self = shift;
    my ($Product) = @_;

    return '' unless $Product->get('SuperProduct');
    return $Product->get('SuperProduct')->alias;
}

sub shipping {
    my $self = shift;
    my ($Product) = @_;

    my $ShippingMethod = $self->shippingMethod;

    return '' unless _productCanUseShippingMethod($Product, $ShippingMethod);

    my $ShippingPrice = $self->shippingPrice($Product);
    $ShippingPrice = '0.0' if $ShippingPrice == 0;

    my $ShippingMethodName = $ShippingMethod->get('Name', $self->languageID);

    return "::$ShippingMethodName:$ShippingPrice " .  $self->currencyID;
}

sub shipping_weight {
    my $self = shift;
    my ($Product) = @_;

    my $Weight = $Product->get('Weight');
    my $WeightUnit = $Product->get('WeightUnit');

    return '' unless $Weight && $WeightUnit;
    
    my $WeightInGrams = $Weight * $WeightUnit->get('Factor');
    return "$WeightInGrams g";
}

sub _productCanUseShippingMethod {
    my ($Product, $ShippingMethod) = @_;

    my $aShippingMethods = $Product->get('ShippingMethods');
    return 1 if scalar @$aShippingMethods == 0;

    foreach my $SMethod ( @$aShippingMethods ) {
        if ( $SMethod->id == $ShippingMethod->id ) {
            return 1;
        }
    }

    return 0;
}

#========================================================================================
# function     exportLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->exportLine;
#----------------------------------------------------------------------------------------
# description  Export one line, increments monitor.
#========================================================================================
sub exportLine {
    my $self = shift;
    my $Object = LoadObject($self->topID('Item'));

    my $Monitor = $self->monitor;
    if (defined $Monitor) {
        $self->monitorIncrement;
        $self->monitorContent;
    }

    #-- Skip Master products
    return if $Object->get('IsMaster');

    #-- Must have a name, price and be visible
    eval {
        $self->isProductValidPortalExt1($Object);
    };
    if (ExistsError()) {
        #-- Just return, don't error, because it will use the PortalConfig object as its servlet view object
        #-- and ours runs on the Shop object, which will not work
        #my $Error = GetError();
        #$Error->addVars({'ProductNotExported' => 1});
        #Warning($Error);
        return;
    }

    $self->writeLine($self->getExportValues($Object));
}

#========================================================================================
# function     isProductValidPortalExt1
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isProductValidPortalExt1;
#----------------------------------------------------------------------------------------
# description  Throws error if not valid (not visible, no name or no price).
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub isProductValidPortalExt1 {
    my $self = shift;
    my ($Product) = @_;

    IsProductValidPortalExt1($self->object, $Product, $self->currencyID, $self->taxModel, $self->languageID);
}

1;
