#==============================================================================
# package      DE_EPAGES::Product::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Product::UI::Shop;
use base DE_EPAGES::Product::UI::ProductFolder;

use strict;
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Order::API::Basket qw (GetBasket);
use DE_EPAGES::Product::API::Basket qw (AddProducts);
use DE_EPAGES::Product::API::Shop qw (GetProduct);
use DE_EPAGES::ShopMailType::API::ShopMailType qw (GetShopMailType);
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Product::API::TopSeller qw ( GetTopSellerForShop );
use DE_EPAGES::Product::API::Constants qw (
    UNITY_CUSTOMIZABLE_HEADLINELENGTH
);

use constant NUMBER_RESULTS => 30;

sub GetAppointmentFolder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Folder = $Servlet->object->folder('Appointments', {'Position'=>'170', 'Class'=>LoadClassByAlias('AppointmentFolder') });
    $Servlet->vars('ObjectID', $Folder->id);
    return $self->ViewPagedChildren($Servlet);
}

sub ViewItemEntryForm {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars('Lines', [
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},

      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
      {'Alias' => '', 'Quantity' => ''},
    ]);
}

sub MultiAddToBasket {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    # validate input data and get unformatted values
    # third parameter = 1 means skipExecuteFormError
    my $hValues = $Servlet->form->form($Shop, 'ItemEntryForm', 1);

    my $idx = -1;
    foreach my $item (@{$hValues->{'Items'}}) {
        $idx++;

        my $alias = $item->{'Alias'} || '';

        # no problem if both are empty
        next if $alias =~ /\S/;
        $Servlet->form->addFormError({
            'Reason'    => 'FieldEmpty',
            'Name'      => 'Alias',
            'LoopName'  => 'Items',
            'Index'     => $idx,
        }) if (not defined $item->{'Alias'} and defined $item->{'Quantity'});
    }

    my $User = $Servlet->user // $Servlet->createAnonymousUser($Shop);
    my $hLocaleInfo = {
        'LocaleID'   => $Servlet->locale,
        'CurrencyID' => $Servlet->currencyID,
    };
    my @Items;
    my %IndexMap;
    $idx = -1;
    my $Index = 1;
    foreach my $item (@{$hValues->{'Items'}}) {
        $idx++;

        my $alias = $item->{'Alias'} || '';
        next unless $alias =~ /\S/;

        # remove space at begin and end
        $alias =~ s/^\s//;
        $alias =~ s/\s$//;

        if (defined $alias) {
            my $Product = GetProduct($Shop, $alias);
            if (defined $Product) {
                # we do not add customizable products from the order list to the basket
                if($Product->get('IsCustomizable')) {
                    $Servlet->form->addFormError({
                        'Reason'    => 'ProductIsCustomizable',
                        'Name'      => 'Alias',
                        'LoopName'  => 'Items',
                        'Index'     => $idx,
                    });
                } else {
                    $IndexMap{$Product->id} = $idx;
                    # add product to basket later
                    push @Items, {
                        'Product' => $Product,
                        'Position' => $Index * 10, ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
                        'Quantity' => defined $item->{'Quantity'} ? $item->{'Quantity'} : $Product->get('MinOrder'),
                    };
                    $Index++;
                }
            } else {
                $Servlet->form->addFormError({
                    'Reason'    => 'AliasNotFound',
                    'Name'      => 'Alias',
                    'LoopName'  => 'Items',
                    'Index'     => $idx,
                });
            }
        }
    }
    $Servlet->form->addFormError({
        'Reason'    => 'ListEmpty',
        'Name'      => 'Alias',
        'LoopName'  => 'Items',
        'Index'     => 0,
    }) unless scalar @Items;

    my $aWarnings = BlockWarnings( sub {
        AddProducts($User, $hLocaleInfo, \@Items);
    }, ['ProductCanNotChangedAtBasket', 'ProductCanNotAddToBasket', 'ProductNoPrice', 'NoPayment', 'NoShipping' ]);

    foreach my $Warning (@$aWarnings) {
        if ($Warning->code eq 'NoShipping' && $Shop->featureMaxValue('ShippingMethods') == 0) {
            # this is a shop without shipping feature: ignore error
        } elsif ($Warning->code eq 'NoPayment' && $Shop->featureMaxValue('PaymentMethods') == 0) {
            # this is a shop without payment feature: ignore error
        } else {
            $Servlet->form->addFormError({
                'Reason'    => $Warning->code,
                'Name'      => 'Alias',
                'LoopName'  => 'Items',
                'Index'     => $IndexMap{$Warning->vars->{'ProductID'}},
            });
        }
    }
    $Servlet->form->executeFormError();


    # display last view only if we have a LastViewAction and the correct setting for AddToBasketView
    if( !$Shop->get('AddToBasketView') && $Servlet->params('LastViewAction') ) {
        $Servlet->vars('ViewAction', $Servlet->params('LastViewAction'));
    } else {
        my $Basket = GetBasket($User, $Shop, $hLocaleInfo);
        $Servlet->vars('ViewObjectID' => $Basket->id) if defined $Basket;
    }

    return;
}

sub DeleteAllProducts {
    my $self = shift;
    my ($Servlet) = @_;

    # delete only the user confirmed the accept message
    if ($Servlet->params('ACCEPT')) {
        $Servlet->vars('DELETEALL',0);
    } else {
        $Servlet->vars('DELETEALL',1);
        return;
    }

    # now delete all objects
    my $Folder = $Servlet->object()->folder('Products');
    RunBatchProcess('DeleteAllProducts', sub {
        my $ProductIterator = $Folder->childrenIterator();
        while (my $Product = <$ProductIterator>) {
            $Product->delete;
        }
    });
    return;
}

sub ViewBookingsystemWizard{
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $Folder = $Shop->folder('MailTypes');
    foreach my $key ( qw (AppointmentCreated AppointmentAccepted AppointmentCancelled)){
        if(not $Folder->existsChild( $key ) ) {
            my $MailType = GetShopMailType($Shop,$key);
            $MailType->set({'IsActive'=>1});
        }
    }
    $Shop->businessHours;
    return $self->SUPER::View($Servlet);
}

sub SaveProductSettings{
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->form->form( $Servlet->object, 'ProductSettings' );

    return DE_EPAGES::Presentation::UI::Object::Save($self, $Servlet);
}


sub SaveProductStockSettings{
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    # check TLE Syntax by using the form handling
    my $Form = $Servlet->form;
    $Form->form($Shop, 'SaveProductStockSettings');

    $self->DE_EPAGES::Presentation::UI::Object::Save($Servlet);
    $Shop->updatePageCache;

    return;
}

#========================================================================================
# function     ViewTopSeller
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewTopSeller($Servlet);
#----------------------------------------------------------------------------------------
# description  Get TopSeller statistic from cached table and set result
#               as JSON string to #INPUT.TopSellerJSON. If no TopSeller is available
#               set #INPUT.TopSellerJSON undef.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub ViewTopSeller {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $hTopSeller = GetTopSellerForShop($Shop, $Servlet->languageID(), NUMBER_RESULTS);
    $Servlet->params('TopSellerJSON', $hTopSeller->{Count} > 0 ? $hTopSeller->{JSON} : undef );

    return;
}

sub UnityViewStockSettings {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->SUPER::View($Servlet);
}

sub UnityViewProductSettings {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $Servlet->vars('CustomizableHeadlineLength', UNITY_CUSTOMIZABLE_HEADLINELENGTH);

    $self->SUPER::View($Servlet);
}

1;
