#========================================================================================
# package      DE_EPAGES::ProductPortal::API::Export
# state        public
# description  API functions needed for export of portal products.
#========================================================================================
package DE_EPAGES::ProductPortal::API::Export;
use base Exporter;

use strict;
use DE_EPAGES::Core::API::Error      qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Shop::API::Url        qw (BuildShopUrl);
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

our @EXPORT_OK  = qw (
    FormatPrice
    GetImageUrl
    IsProductValidPortal
    IsProductValidPortalExt1
    GetProductPrice
    GetProductUrl
    GetShippingPrice
);

sub ErrorDefinition {
    return {
        'ProductNotVisible' => 'product ( #ProductID ) is not visible, can not export to portal.',
        'ProductNoName'     => 'product ( #ProductID ) has no name, can not export to portal.',
        'ProductNoPrice'    => 'product ( #ProductID ) has no price, can not export to portal.',
        'ProductNoShipping' => 'product ( #ProductID ) has no shipping price, can not export to portal.'
    }
}

#========================================================================================
# function     IsProductValidPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsProductValidPortal($Product);
#----------------------------------------------------------------------------------------
# description  Throws error if not valid, (not visible).
#----------------------------------------------------------------------------------------
# input        $Product | product | object
#========================================================================================
sub IsProductValidPortal {
    my ($Product) = @_;

    #test for visible
    unless ($Product->get('IsVisible')) {
        Error('ProductNotVisible', {'ProductID'=>$Product->id, 'Product'=>$Product});
    }
    return;
}

#========================================================================================
# function     IsProductValidPortalExt1
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsProductValidPortalExt1($PortalConfig, $Product);
#----------------------------------------------------------------------------------------
# description  Throws error if not valid (not visible, no name or no price).
#----------------------------------------------------------------------------------------
# input        $PortalConfig | portal config | object
# input        $Product | product | object
# input        $CurrencyID | currency id (optional) | string
# input        $TaxModel | tax model (optional) | 0/1
# input        $LanguageID | language id to use for the check (optional) | integer
#========================================================================================
sub IsProductValidPortalExt1 {
    my ($PortalConfig, $Product, $CurrencyID, $TaxModel, $LanguageID) = @_;

    #test for visible
    Error('ProductNotVisible', {'ProductID'=>$Product->id, 'Product'=>$Product}) unless $Product->get('IsVisible');

    my $LanguageIDToUse = defined $LanguageID ? $LanguageID : $PortalConfig->get('LanguageID');

    #test for name
    Error('ProductNoName', {'ProductID'=>$Product->id, 'Product'=>$Product}) unless defined $Product->getWithParent('Name', $LanguageIDToUse);

    #Product->price throw error ProductNoPrice
    $CurrencyID //= $PortalConfig->get('CurrencyID');
    $TaxModel   //= $PortalConfig->get('PortalSite')->get('TaxModel');
    eval {
        GetProductPrice($Product, $CurrencyID, $TaxModel);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Error->addVars({ 'Product' => $Product }) if $Error->code eq 'ProductNoPrice';
        Error($Error);
    }

    # test if a shipping method is available for the product
    if ($PortalConfig->get('PortalSite')->get('ExportsShippingPrice')) {
        my $ShippingMethod = $PortalConfig->shippingMethodForProduct($Product);
        if (!defined $ShippingMethod) {
            Error('ProductNoShipping', {'ProductID' => $Product->id, 'Product' => $Product});
        }
    }
    return;
}

#========================================================================================
# function     GetProductPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Price = GetProductPrice($Product, $CurrencyID, $TaxModel);
#----------------------------------------------------------------------------------------
# description  get price of product with vars of portal config.
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# input        $CurrencyID | currency id | string
# input        $TaxModel | tax model | 0/1
# return       $Price | product price | float
#========================================================================================
sub GetProductPrice {
    my ($Product, $CurrencyID, $TaxModel) = @_;

    my $hVars = {
        'CurrencyID' => $CurrencyID,
        'TaxModel'   => $TaxModel
    };
    my $ecoTax = $Product->getEcoParticipation($CurrencyID, $TaxModel) // 0;

    return $Product->price($hVars) + $ecoTax - $Product->discount($hVars);
}

#========================================================================================
# function     GetShippingPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = GetShippingPrice($PortalConfig, $Product, $CurrencyID, $TaxModel);
#----------------------------------------------------------------------------------------
# description  get shipping price for product with vars of portal config.
#----------------------------------------------------------------------------------------
# input        $PortalConfig | portal config | object
# input        $Product | product | object
# input        $CurrencyID | currency id (optional) | string
# input        $TaxModel | tax model (optional) | 0/1
# return       $Price | shipping price | float
#========================================================================================
sub GetShippingPrice {
    my ($PortalConfig, $Product, $CurrencyID, $TaxModel) = @_;

    return 0 if $Product->get('IsDownloadProduct');
    if ($Product->get('IsBundleProduct')) {
        my $IsDownloadOnly = 1;
        foreach my $BundledProduct (@{$Product->get('BundledProducts')}) {
            $IsDownloadOnly = 0 unless $BundledProduct->get('IsDownloadProduct');
        }
        return 0 if $IsDownloadOnly;
    }

    my $ShippingMethod = $PortalConfig->shippingMethodForProduct($Product);
    if (!defined $ShippingMethod) {
        Error('ProductNoShipping', {'ProductID' => $Product->id});
    }

    $CurrencyID //= $PortalConfig->get('CurrencyID');
    $TaxModel   //= $PortalConfig->get('PortalSite')->get('TaxModel');
    my $hVars = {
        'CurrencyID'=> $CurrencyID,
        'TaxModel'  => $TaxModel,
        'ShippingElements' => [{
            'Element'  => $Product ,
            'Quantity' => $Product->getWithParent('MinOrder')
        }]
    };
    my $ShippingMethodPrice;
    my $ShippingMethodDiscount;
    eval {
        $ShippingMethodPrice    = $ShippingMethod->price($hVars);
        $ShippingMethodDiscount = $ShippingMethod->discount($hVars);
    };
    if (ExistsError()) {
        my $Error = GetError();
        #EPG-24785 If there is no price there is also no shipping
        if ( $Error->code eq 'ProductNoPrice' ){ Error('ProductNoShipping'); }
        Error($Error);
    }
    return $ShippingMethodPrice - $ShippingMethodDiscount;
}

#========================================================================================
# function     FormatPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedValue = FormatPrice($Value, $PortalSite, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  formats prices (from shipping or products) to portal specific format.
#----------------------------------------------------------------------------------------
# input        $Value | value to format | float
# input        $PortalSite | portal site | object
# input        $CurrencyID | currency id | string
# return       $FormattedValue | formatted value | string
#========================================================================================
sub FormatPrice {
    my ($Value, $PortalSite, $CurrencyID) = @_;

    my $FormatAttributes = $PortalSite->formatAttributes($CurrencyID);
    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new(
        'decimal_point'  => $FormatAttributes->{'DecimalSep'},
        'thousands_sep'  => $FormatAttributes->{'ThousandSep'},
        'neg_format'     => $FormatAttributes->{'NegFormatNumbers'},
        'decimal_digits' => $FormatAttributes->{'MinorUnitCurrency'},
        'decimal_fill'   => 1
    );
    return $Formatter->format_number($Value);
}

#========================================================================================
# function     GetProductUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URL = GetProductUrl($Product);
#----------------------------------------------------------------------------------------
# description  Returns url to product.
#----------------------------------------------------------------------------------------
# input        $PortalConfig | portal config | object
# input        $Product | product | object
# input        $Add2Basket | use add to basket in url (optional) | boolean
# input        $CurrencyID | currency id (optional) | string
# return       $URL | product url | string
#========================================================================================
sub GetProductUrl {
    my ($PortalConfig, $Product, $Add2Basket, $CurrencyID) = @_;

    $Add2Basket //= $PortalConfig->get('Add2Basket');
    $CurrencyID //= $PortalConfig->get('CurrencyID');

    my %Params = (
        'ViewAction' => 'ViewProductViaPortal',
        'Locale'     => $PortalConfig->get('PortalSite')->get('LocaleID')
    );

    if ($Add2Basket) {
        $Params{'ChangeAction'} = 'AddToBasket';
        $Params{'ViewProduct'}  = 1 ;
    }

    # if currency differs from shop's default currency => export
    my $ShopCurrencyID = $Product->getSite()->get('CurrencyID');
    $Params{'Currency'} = $CurrencyID if $CurrencyID ne $ShopCurrencyID;

    my $URL;
    my $UseSSL = $Product->getSite()->get('HasSSLCertificate');
    $URL = BuildShopUrl($Product, \%Params, { 'UseObjectPath' => 1, 'UseSSL' => $UseSSL });

    # add extensions to url, e.g. etracker parameter
    my $hParams = {
        'PortalConfig' => $PortalConfig,
        'Url'          => $URL
    };
    TriggerHook('PortalExportProductUrl', $hParams);

    return $hParams->{'Url'};
}

#========================================================================================
# function     GetImageUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URL = GetImageUrl($Product, $ImageAttribute);
# example      $URL = GetImageUrl($Product, $self->imageAttribute);
#----------------------------------------------------------------------------------------
# description  Returns url to product image.
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# input        $ImageAttribute | image attribute | object
# input        $LanguageID | language id  | int
# return       $URL | image url | string
#========================================================================================
sub GetImageUrl {
    my ($Product, $ImageAttribute, $LanguageID) = @_;

    my $imageUrl;
    if (defined $ImageAttribute) {
        my $Alias = $ImageAttribute->alias;
        my $Value = $Product->getWithParent($Alias, $LanguageID);
        if (defined $Value) {
            if ($Value =~ /\/\//) { #contains //hostname
                $imageUrl= $Value;
            }
            else {
                my $Shop = $Product->getSite();
                my $HasSSLCertificate = $Shop->get('HasSSLCertificate');
                my $ProtocolServerName = ($HasSSLCertificate) ?
                    $Shop->get('ProtocolAndServerSSL') :
                    $Shop->get('ProtocolAndServer');
                $imageUrl = $ProtocolServerName . $Product->get('WebPath') . '/' . $Value;
            }
        }
    }
    return $imageUrl;
}

1;
