#========================================================================================
# §package      DE_EPAGES::Test::API::TranslationFiles
# §base         DE_EPAGES::Test::API::UnitTest
# §state        public
#----------------------------------------------------------------------------------------
# §description  Checks translation files for correct file structure
#========================================================================================
package DE_EPAGES::Test::API::TranslationFiles;
use base qw (Exporter DE_EPAGES::Test::API::UnitTest);

use strict;

use Mojo::DOM;
use File::Slurper 'read_text';

use Test::More;
use DE_EPAGES::Core::API::File qw (
    GetFileNames
    GetFileContentLines
    GetSubDirectories
);

our @EXPORT_OK = qw(
    TestCartridges
    TestCompany
    TestTranslationFile
);

sub test {
    my $self = shift;

    # get companies
    my @Companies = _GetCompanies();

    foreach my $Company (@Companies) {
        TestCompany($Company);
    }

    done_testing();
}

#========================================================================================
# §function     TestCompany
#----------------------------------------------------------------------------------------
# §syntax       TestCompany($Company);
#----------------------------------------------------------------------------------------
# §description  Tests all translation files of a company
#----------------------------------------------------------------------------------------
# §input        $Company     | cartridge folder name, e.g. DE_EPAGES | string
#========================================================================================
sub TestCompany {
    my ($Company) = @_;

    # get cartridges
    my @Cartridges = _GetCartridges($Company);

    TestCartridges(\@Cartridges, $Company);
}

#========================================================================================
# §function     TestCartridges
#----------------------------------------------------------------------------------------
# §syntax       TestCartridges($aCartridges, $Company);
#----------------------------------------------------------------------------------------
# §description  Tests all translation files of a cartridge
#----------------------------------------------------------------------------------------
# §input        $aCartridges | list of cartridge names | ref.array
# §input        $Company     | cartridge folder name, e.g. DE_EPAGES | string
#========================================================================================
sub TestCartridges {
    my ($aCartridges, $Company) = @_;

    # get Translation*.xml files
    my @TranslationFiles = map { _GetTranslationFiles($_, $Company) } @$aCartridges;

    foreach my $FileName (@TranslationFiles) {
        TestTranslationFile($FileName);
    }
}


#========================================================================================
# §function     TestTranslationFile
#----------------------------------------------------------------------------------------
# §syntax       TestTranslationFile($FileName);
#----------------------------------------------------------------------------------------
# §description  Tests a single translation file
#----------------------------------------------------------------------------------------
# §input        $FileName | file name (full path) | string
#========================================================================================
sub TestTranslationFile {
    my ($FileName) = @_;

    my $aFileContentLines = GetFileContentLines($FileName, {'encoding' => 'utf-8'});
    # check XML declaration
    is($aFileContentLines->[0], "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n",
        $FileName. ' does start with the correct xml declaration: ' . $aFileContentLines->[0]);

    # check <epages> tags
    ok($aFileContentLines->[1] =~ m/^<(ns0:|)epages( |>)/,
        $FileName. ' does have the opening <epages> tag: ' . $aFileContentLines->[1]);
    ok($aFileContentLines->[-1] =~ m/^<\/(ns0:|)epages>\n/,
        $FileName. ' does have the closing </epages> tag: ' . $aFileContentLines->[-1]);

    # check <Language> tags
    ok($aFileContentLines->[2] =~ m/^ *(<(ns0:|)Language Language="[a-z]{2}">)\n/
        || $aFileContentLines->[3] =~ m/^ *(<(ns0:|)Language Language="[a-z]{2}">)\n/,
        $FileName. ' does have a correct opening <Language> tag: ' . $aFileContentLines->[2]);
    ok($aFileContentLines->[-2] =~ m/^\s*<\/(ns0:|)Language>\n/,
        $FileName. ' does have a correct closing </Language> tag: ' . $aFileContentLines->[-2]);

    # check for singleton <Translation> tags
    my $FileContent = join '', @$aFileContentLines;
    ok($FileContent !~ m/(<Translation[^>]*\/>)/,
        $FileName. ' contains no singleton <Translation /> tag: ' . $1);

    # check for singleton <notrans /> tags
    ok($FileContent !~ m/(<notrans *\/>)/,
        $FileName. ' contains no singleton <notrans /> tag: ' . $1);

    # check for notransed whitespaces, <notrans> <notrans /> tags
    ok($FileContent !~ m/(<notrans[^>]*>\s*<\/ *notrans>)/,
        $FileName. ' contains no whitespace <notrans> tag: ' . $1);

    # check for untrimmed text
    ok($FileContent !~ m/<Translation[^>]*>(\s+[^\s<>]+.*)<\/Translation>/
        && $FileContent !~ m/<Attribute[^>]*>(\s+[^\s<>]+.*)<\/Attribute>/,
        $FileName. ' has no leading whitespaces: ' . $1);
    ok($FileContent !~ m/<Translation[^>]*>(.*[^\s<>]+\s+)<\/Translation>/
        && $FileContent !~ m/>Attribute[^>]*>(.*[^\s<>]+\s+)<\/Attribute>/,
        $FileName. ' has no trailing whitespaces: ' . $1);

    # check for XML junk
    ok($FileContent !~ m/<\/Translation>(\s*[^\s<>]+\s*)</
        && $FileContent !~ m/<\/Attribute>(\s*[^\s<>]+\s*)</,
        $FileName. ' has no junk between XML tags: ' . $1);
    ok($FileContent !~ m/[\x{007f}-\x{009f}]/,
        $FileName. ' does not include awkward whitespaces');
    ok($FileContent !~ m/<!\[CDATA\[/,
        $FileName. ' does not include \'<![CDATA[\' tags');

    # check for wrong indentations
    ok($FileContent !~ m/\t/, $FileName. ' contains no tabulator');

    # check if XML tags contain necessary attributes
    my $xml = read_text $FileName;
    my $dom = Mojo::DOM->new->xml(1)->parse($xml);
    my $translationTags = $dom->find('Translation');
    my $objectTags = $dom->find('Object');
    my $attributeTags = $dom->find('Attribute');

    foreach my $tag (@$translationTags) {
        # check if <Language> tags contain 'Keyword'
        ok($tag =~ m/^<Translation (?=.*Keyword="[\w]+")/,
            $FileName. ' has the <Translation> tag, which includes a non-empty \'Keyword\' attribute: ' . $tag);
    }

    foreach my $tag (@$objectTags) {
        # check if <Object> tags contain 'Path'
        ok($tag =~ m/^<(ns0:|)Object (?=.*Path="[^"]+")/,
            $FileName. ' has the <Object> tag, which includes a non-empty \'Path\' attribute: ' . $tag);
    }

    foreach my $tag (@$attributeTags) {
        # check if <Attribute> tags contain 'Name'
        ok($tag =~ m/^<(ns0:|)Attribute (?=.*Name="[\w]+")[^>]*>/,
            $FileName. ' has the <Attribute> tag, which includes a non-empty \'Name\' attribute: ' . $tag);
    }
}

#----------------------------------------------------------------------------------------
# private helper
#----------------------------------------------------------------------------------------
sub _GetTranslationFiles {
    my ($CartridgeName, $Company) = @_;

    my @Dirlist = (
        join ('/', $ENV{'EPAGES_CARTRIDGES'}, $Company, $CartridgeName, 'Templates'),
        join ('/', $ENV{'EPAGES_CARTRIDGES'}, $Company, $CartridgeName, 'Database'),
        join ('/', $ENV{'EPAGES_CARTRIDGES'}, $Company, $CartridgeName, 'Data'),
        );

    # file exceptions that seem to be translation files but are not
    my @FileNameExceptions = (
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Database/XML/Languages.en-US.xml',
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Database/XML/Languages.es-MX.xml',
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Database/XML/Languages.pt-BR.xml'
    );

    return  grep { !($_ ~~ @FileNameExceptions) }
            grep { /[.](?:\w{2}|[\w-]{5})[.]xml$/x }
            map  { @{GetFileNames($_, {'recursive' => 1})} }
            grep { -d $_ } @Dirlist;
}

sub _GetCartridges {
    my ($Company) = @_;

    my $aCartridges = GetSubDirectories($ENV{'EPAGES_CARTRIDGES'} . '/' . $Company, {'relative' => 1});
    my @ExcludedCartridges;
    @ExcludedCartridges = (
        'DocdataPay',
        'ELogistics',
        'Envialia',
        'TagCommander',
        'MultiStore',
        'MultiStoreCommon',
        'MultiStoreConnector',
        'MultiStoreMaster',
        'MultiStoreMasterConnector',
        'MultiStoreProvider',
        'MultiStoreShipping'
    ) if $Company eq 'DE_EPAGES';

    return grep { !($_ ~~ @ExcludedCartridges) } @$aCartridges;
}

sub _GetCompanies {
    my $aCompanies = GetSubDirectories($ENV{'EPAGES_CARTRIDGES'}, {'relative' => 1});

    my @ExcludedCompanies = (
        'CPAN',
        'RND_EPAGES',
        '.git'
    );

    return grep { !($_ ~~ @ExcludedCompanies) } @$aCompanies;
}

1;
